/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.gui;

import jalview.viewmodel.OverviewDimensions;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;

import javax.swing.JPanel;

/**
 * Panel displaying an overview of the full alignment, with an interactive box
 * representing the viewport onto the alignment.
 * 
 * @author $author$
 * @version $Revision$
 */
public class OverviewPanel extends JPanel implements Runnable
{
  private OverviewDimensions od;

  private OverviewCanvas oviewCanvas;

  private AlignViewport av;

  private AlignmentPanel ap;

  /**
   * Creates a new OverviewPanel object.
   * 
   * @param alPanel
   *          The alignment panel which is shown in the overview panel
   */
  public OverviewPanel(AlignmentPanel alPanel)
  {
    this.av = alPanel.av;
    this.ap = alPanel;

    od = new OverviewDimensions(av.getRanges(),
            (av.isShowAnnotation() && av
                    .getAlignmentConservationAnnotation() != null));

    oviewCanvas = new OverviewCanvas(od, av);
    setLayout(new BorderLayout());
    add(oviewCanvas, BorderLayout.CENTER);

    addComponentListener(new ComponentAdapter()
    {
      @Override
      public void componentResized(ComponentEvent evt)
      {
        if ((getWidth() != od.getWidth())
                || (getHeight() != (od.getHeight())))
        {
          updateOverviewImage();
        }
      }
    });

    addMouseMotionListener(new MouseMotionAdapter()
    {
      @Override
      public void mouseDragged(MouseEvent evt)
      {
        if (!av.getWrapAlignment())
        {
          od.updateViewportFromMouse(evt.getX(), evt.getY(), av
                  .getAlignment().getHiddenSequences(), av
                  .getColumnSelection(), av.getRanges());
          ap.setScrollValues(od.getScrollCol(), od.getScrollRow());
        }
      }
    });

    addMouseListener(new MouseAdapter()
    {
      @Override
      public void mousePressed(MouseEvent evt)
      {
        if (!av.getWrapAlignment())
        {
          od.updateViewportFromMouse(evt.getX(), evt.getY(), av
                  .getAlignment().getHiddenSequences(), av
                  .getColumnSelection(), av.getRanges());
          ap.setScrollValues(od.getScrollCol(), od.getScrollRow());
        }
      }
    });

    updateOverviewImage();
  }

  /**
   * Updates the overview image when the related alignment panel is updated
   */
  public void updateOverviewImage()
  {
    if ((getWidth() > 0) && (getHeight() > 0))
    {
      od.setWidth(getWidth());
      od.setHeight(getHeight());
    }

    setPreferredSize(new Dimension(od.getWidth(), od.getHeight()));

    if (oviewCanvas.restartDraw())
    {
      return;
    }

    Thread thread = new Thread(this);
    thread.start();
    repaint();

  }

  @Override
  public void run()
  {
    oviewCanvas.draw(av.isShowSequenceFeatures(),
            (av.isShowAnnotation() && av
                    .getAlignmentConservationAnnotation() != null), ap
                    .getSeqPanel().seqCanvas.getFeatureRenderer());
    setBoxPosition();
  }

  /**
   * Update the overview panel box when the associated alignment panel is
   * changed
   * 
   */
  public void setBoxPosition()
  {
    od.setBoxPosition(av.getAlignment()
            .getHiddenSequences(), av.getColumnSelection(), av.getRanges());
    repaint();
  }
}
